@@ -3,5 +3,5 @@
   library.dynam(pack, pack, lib)
   packageStartupMessage("\nDEoptim package",
    "\nDifferential Evolution algorithm in R",
-   "\nAuthors: David Ardia and Katharine Mullen\n")
+   "\nAuthors: D. Ardia, K. Mullen, B. Peterson and J. Ulrich\n")
 }
